package com.xkrs.dao;

import com.xkrs.model.entity.FireAndRanger;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

/**
 * @author xkrs
 */
@Component
public interface FireAndRangerDao extends JpaRepository<FireAndRanger,Long>, JpaSpecificationExecutor<FireAndRanger> {

    /**
     * 根据火点编码查询信息
     * @param fireCode
     * @return
     */
    FireAndRanger findByFireCode(String fireCode);

    /**
     * 根据护林员手机号查询
     * @param rangerName
     * @param progressType
     * @return
     */
    FireAndRanger findByRangerNameAndProgressType(String rangerName, String progressType);

    /**
     * 根据火点编码删除火点信息
     * @param fireCode
     * @return
     */
    FireAndRanger deleteByFireCode(String fireCode);

    /**
     * 根据火点编码修改结束时间
     * @param fireCode
     * @param endTime
     */
    @Modifying(clearAutomatically=true)
    @Query(value = "update fire_and_ranger set end_time = ?2 where fire_code = ?1",nativeQuery = true)
    void updateEndTimeByFireCode(String fireCode, String endTime);

    /**
     * 根据火点编码修改进程状态
     * @param fireCode
     * @param endTime
     */
    @Modifying(clearAutomatically=true)
    @Query(value = "update fire_and_ranger set progress_type = ?2 where fire_code = ?1",nativeQuery = true)
    void updateProgressTypeByFireCode(String fireCode, String endTime);
}
